@@ -1,13 +1,13 @@
 import os
 import sqlite3
 
-from xklb.utils import stop
+from xklb.utils import stop, log
 
 
 def sqlite_con(db):
     if not os.path.exists(db):
-        print(f"Database file '{db}' does not exist. Create one with lb extract.")
-        stop()
+        log.error(f"Database file '{db}' does not exist. Create one with lb extract.")
+        exit(1)
 
     con = sqlite3.connect(db)
     con.row_factory = sqlite3.Row
