@@ -96,14 +96,14 @@ def argparse_log():
 log = argparse_log()
 
 
-def mv_to_keep_folder(args, video: Path):
-    kp = re.match(args.shallow_organize + "(.*?)/", str(video))
+def mv_to_keep_folder(args, video):
+    kp = re.match(args.shallow_organize + "(.*?)/", video)
     if kp:
         keep_path = Path(kp[0], "keep/")
-    elif video.parent.match("*/keep/*"):
+    elif Path(video).parent.match("*/keep/*"):
         return
     else:
-        keep_path = video.parent / "keep/"
+        keep_path = Path(video).parent / "keep/"
 
     keep_path.mkdir(exist_ok=True)
     shutil.move(video, keep_path)
@@ -255,7 +255,9 @@ def compile_query(query, *args):
     number_of_question_marks = query.count("?")
     number_of_arguments = len(args)
     if number_of_arguments != number_of_question_marks:
-        return f"Number of bindings mismatched. The query uses {number_of_question_marks}, but {number_of_arguments} parameters bound."
+        raise Exception(
+            f"Number of bindings mismatched. The query uses {number_of_question_marks}, but {number_of_arguments} parameters bound."
+        )
 
     for a in args:
         query = query.replace("?", "'" + str(a) + "'", 1)
@@ -267,7 +269,7 @@ def print_query(query, bindings):
     return re.sub(r"\n\s+", r"\n", compile_query(query, *bindings))
 
 
-def single_column_tolist(array_to_unpack, column_name=1):
+def single_column_tolist(array_to_unpack, column_name: str | int = 1):
     return list(
         map(
             lambda x: x[column_name],
@@ -276,7 +278,7 @@ def single_column_tolist(array_to_unpack, column_name=1):
     )
 
 
-def get_ordinal_media(args, path: Path):
+def get_ordinal_media(args, path):
     similar_videos = []
     candidate = str(path)
 
@@ -298,8 +300,9 @@ def get_ordinal_media(args, path: Path):
 
         candidate = new_candidate
         query = f"""SELECT path FROM media
-            WHERE path like ?
-                and {'1=1' if (args.play_in_order > 2) else args.sql_filter}
+            WHERE 1=1
+                and path like ?
+                {'' if (args.play_in_order > 2) else args.sql_filter}
             ORDER BY path
             LIMIT 1000
             """
@@ -358,7 +361,7 @@ def safe_unpack(*list_, idx=0):
 class argparse_dict(argparse.Action):
     def __call__(self, parser, args, values, option_string=None):
         try:
-            d = dict(map(lambda x: x.split("="), values))
+            d = dict(map(lambda x: x.split("="), values.split(" ")))
         except ValueError as ex:
             raise argparse.ArgumentError(self, f'Could not parse argument "{values}" as k1=1 k2=2 format')
         setattr(args, self.dest, d)
